import numpy as np
import matplotlib.pyplot as plt
from eqsig import multiple
from eqsig import Signal


def show_same_start():
    time = np.linspace(0, 102, 10200)
    acc = np.sin(time)
    dt = 0.01

    cluster = multiple.Cluster([acc, acc + 0.3], dt=dt)
    plt.plot(cluster.time, cluster.values_by_index(0), label="sig0")
    plt.plot(cluster.time, cluster.values_by_index(1), label="sig1")
    cluster.same_start()
    plt.plot(cluster.time, cluster.values_by_index(0), label="sig0 - same start")
    plt.plot(cluster.time, cluster.values_by_index(1), label="sig1 - same start")
    plt.legend()
    plt.show()


def show_time_match():
    time = np.linspace(0, 102, 1020)
    acc = np.sin(time)
    dt = 0.01

    cluster = multiple.Cluster([acc[:-6], acc[6:]], dt=dt)
    plt.plot(cluster.time, cluster.values_by_index(0), label="sig0")
    plt.plot(cluster.time, cluster.values_by_index(1), label="sig1")
    cluster.time_match(verbose=1)
    plt.plot(cluster.time, cluster.values_by_index(0), label="sig0 - same start")
    plt.plot(cluster.time, cluster.values_by_index(1), label="sig1 - same start")
    plt.legend()
    plt.show()


def run_combine():
    import matplotlib.pyplot as plt
    time = np.linspace(0, 100, 1000)
    w = 0.2
    amp = 1.0
    x = amp * np.sin(w * time)
    y = amp / 2 * np.cos(w * time)

    off_rad = np.radians(45.)
    adj = x * np.cos(off_rad) + y * np.sin(off_rad)
    adj_alt = y * np.cos(off_rad) - x * np.sin(off_rad)
    off_rad = np.radians(90.)
    adj2 = x * np.cos(off_rad) + y * np.sin(off_rad)
    plt.plot(x, y)
    plt.plot(adj, adj_alt, ls="--")
    plt.show()
    plt.plot(time, x)
    plt.plot(time, y)
    plt.plot(time, adj)
    plt.plot(time, adj2, ls="--")
    plt.show()


if __name__ == '__main__':
    run_combine()



import numpy as np
import scipy
from eqsig.single import Signal, AccSignal


def time_series_from_motion(motion, dt):
    npts = len(motion)
    return np.linspace(0, dt * (npts + 1), npts)


def determine_indices_of_peaks(values):
    """
    Determines the position of values that form a local peak in a signal.

    :param values:
    :return:
    """
    diff = np.diff(values)
    # if negative then direction has switched
    direction_switch = diff[1:] * diff[:-1]
    direction_switch = np.insert(direction_switch, 0, 0)
    peaks = np.where(direction_switch < 0)
    peak_indices = peaks[0]
    peak_indices = np.insert(peak_indices, 0, 0) # Include first and last value
    peak_indices = np.insert(peak_indices, len(peak_indices), len(values) - 1)

    return peak_indices


# def determine_peaks_only_series(values):
#     diff = np.diff(values)
#     # if negative then direction has switched
#     direction_switch = diff[1:] * diff[:-1]
#     peaks = np.where(direction_switch < 0, 1, 0)
#     # peaks array is shorter by two, indices refer +1, add zeros at start and end
#     peaks = np.insert(peaks, 0, 0)
#     peaks = np.insert(peaks, len(peaks), 0)  # don't use -1 here
#     return values * peaks


def determine_peak_only_series_4_cleaned_data(values):
    """
    Determines the

    Note: array must not contain adjacent repeated values

    :param values:
    :return:
    """
    peak_indices = determine_indices_of_peaks(values)
    peak_values = np.take(values, peak_indices)
    signs = np.where(np.mod(np.arange(len(peak_values)), 2), -1, 1)
    delta_peaks = np.where(-signs * peak_values < 0, -np.abs(peak_values), np.abs(peak_values))
    delta_peaks_series = np.zeros_like(values)
    np.put(delta_peaks_series, peak_indices, delta_peaks)

    return delta_peaks_series


def determine_peak_only_delta_series_4_cleaned_data(values):
    """
    Determines the

    Note: array must not contain adjacent repeated values

    :param values:
    :return:
    """
    peak_indices = determine_indices_of_peaks(values)
    peak_values = np.take(values, peak_indices)
    delta_peaks = np.diff(peak_values)
    delta_peaks = np.insert(delta_peaks, 0, 0)
    delta_peaks_series = np.zeros_like(values)
    assert len(delta_peaks) == len(peak_indices)
    np.put(delta_peaks_series, peak_indices, delta_peaks)

    return delta_peaks_series


def clean_out_non_changing(values):
    """
    Takes an array removes all values that are the same as the previous value.

    :param values: array of floats
    :return: cleaned array, indices of clean values in original array
    """
    diff_values = np.diff(values)
    diff_values = np.insert(diff_values, 0, values[0])
    non_zero_indices = np.where(diff_values != 0)[0]
    non_zero_indices = np.insert(non_zero_indices, 0, 0)

    cleaned_values = np.take(values, non_zero_indices)
    return cleaned_values, non_zero_indices


def determine_peaks_only_delta_series(values):
    """
    Creates an array with the changes between peak values and zeros for non-peak values.

    Parameters
    ----------
    :param values: array_like, array of values
    :return:

    Examples
    --------
    >>> values = np.array([0, 2, 1, 2, 0, 1, 0, -1, 0, 1, 0])
    np.array([0, 2, 1, 2, 0.3, 1, 0.3, -1, 0.4, 1, 0])
    >>> determine_peaks_only_delta_series(values)
    array([0,  2, -1,  1,  0,  -1,  0,  -2,  0,  2,  0])
    """
    # enforce array type
    values = np.array(values)
    # rebase to zero as first value
    values -= values[0]
    # remove all non-changing values
    cleaned_values, non_zero_indices = clean_out_non_changing(values)
    cleaned_values *= np.sign(cleaned_values[1])  # ensure first value is increasing
    # compute delta peaks for cleaned data
    cleaned_delta_peak_series = determine_peak_only_delta_series_4_cleaned_data(cleaned_values)
    # re-index data to uncleaned array
    delta_peaks_series = np.zeros_like(values)
    np.put(delta_peaks_series, non_zero_indices, cleaned_delta_peak_series)
    return delta_peaks_series


def determine_pseudo_cyclic_peak_only_series(values):
    """
    Creates an array with only peak values assuming an alternative sign and zeros for non-peak values.

    Parameters
    ----------
    :param values: array_like, array of values
    :return:

    Examples
    --------
    >>> values = np.array([0, 2, 1, 2, 0, 1, 0, -1, 0, 1, 0])
    np.array([0, 2, 1, 2, 0.3, 1, 0.3, -1, 0.4, 1, 0])
    >>> determine_pseudo_cyclic_peak_only_series(values)
    array([0,  2, -1,  2,  0,  1,  0,  1,  0,  1,  0])
    """
    # enforce array type
    values = np.array(values)
    # rebase to zero as first value
    values -= values[0]
    # remove all non-changing values
    cleaned_values, non_zero_indices = clean_out_non_changing(values)
    cleaned_values *= np.sign(cleaned_values[1])  # ensure first value is increasing
    # compute delta peaks for cleaned data
    cleaned_delta_peak_series = determine_peak_only_series_4_cleaned_data(cleaned_values)
    # re-index data to uncleaned array
    delta_peaks_series = np.zeros_like(values)
    np.put(delta_peaks_series, non_zero_indices, cleaned_delta_peak_series)
    return delta_peaks_series


def fas2signal(fas, dt, stype="signal"):
    """
    Convert a fourier spectrum to time series signal

    :param fas: positive part only
    :param dt: time step of time series
    :return:
    """
    n = 2 * len(fas)
    a = np.zeros(2 * len(fas), dtype=complex)
    a[1:n // 2] = fas[1:]
    a[n // 2 + 1:] = np.flip(np.conj(fas[1:]), axis=0)
    a /= dt
    s = np.fft.ifft(a)
    npts = int(2 ** (np.log(n) / np.log(2)))
    s = s[:npts]
    if stype == 'signal':
        return Signal(s, dt)
    else:
        return AccSignal(s, dt)


def generate_fa_spectrum(sig):
    npts = len(sig.values)
    n_factor = 2 ** int(np.ceil(np.log2(npts)))
    fa = scipy.fft(sig.values, n=n_factor)
    points = int(n_factor / 2)
    assert len(fa) == n_factor
    fa_spectrum = fa[range(points)] * sig.dt
    fa_frequencies = np.arange(points) / (2 * points * sig.dt)
    return fa_spectrum, fa_frequencies



# Spectrum values
# Author: Maxim Millen
#
# takes values from modal analysis and uses the NZS 1170.5 design code to get the accelerations

import numpy as np


def c_h_factor(period, site_class="C"):

    single = 0
    if isinstance(period, float):
        single = 1
        period = [period]
    c_h_values = np.zeros(len(period))
    for i in range(len(period)):
        tt = period[i]
        if tt < 0:
            print('Structural period is negative')
            raise ValueError
        else:
            if site_class == 'C':
                if tt == 0:
                    ch_factor = 1.33
                elif tt < 0.1:
                    ch_factor = 1.33 + 1.60 * (tt / 0.1)
                elif tt < 0.3:
                    ch_factor = 2.93
                elif tt < 1.5:
                    ch_factor = 2.0 * (0.5 / tt) ** 0.75
                elif tt < 3.0:
                    ch_factor = 1.32 / tt
                else:
                    ch_factor = 3.96 / tt ** 2
            elif site_class == 'D':
                if tt == 0:
                    ch_factor = 1.12
                elif tt < 0.1:
                    ch_factor = 1.12 + 1.88 * (tt / 0.1)
                elif tt < 0.56:
                    ch_factor = 3.0
                elif tt < 1.5:
                    ch_factor = 2.4 * (0.75 / tt) ** 0.75
                elif tt < 3.0:
                    ch_factor = 2.14 / tt
                else:
                    ch_factor = 6.42 / tt ** 2
            elif site_class == 'E':
                if tt == 0:
                    ch_factor = 1.12
                elif tt < 0.1:
                    ch_factor = 1.12 + 1.88 * (tt / 0.1)
                elif tt < 1.0:
                    ch_factor = 3.0
                elif tt < 1.5:
                    ch_factor = 3.0 / tt ** 0.75
                elif tt < 3.0:
                    ch_factor = 3.32 / tt
                else:
                    ch_factor = 9.96 / tt ** 2
            else:
                print('Soil must be type C, D or E')
                raise ValueError
            c_h_values[i] = ch_factor

        if single:
            c_h_values = c_h_values[0]
    return c_h_values


def t_eff(displacement, site_class, z_factor, r_factor, n_factor):
    """
    Returns the effective period based on the displacement and using the NZ design Spectrum
    :param displacement:
    :param site_class:
    :param z_factor:
    :param r_factor:
    :param n_factor:
    """
    # Assume N (near fault factor) is 1.0
    # given C=Ch*R*N*Z
    gravity = 9.81
    if site_class == 'C':
        t_c = 3.0
        d_c = 3.96 * z_factor * r_factor * n_factor / (2 * np.pi) ** 2 * gravity
    elif site_class == 'D':
        t_c = 3.0
        d_c = 6.42 * z_factor * r_factor * n_factor / (2 * np.pi) ** 2 * gravity
    elif site_class == 'E':
        t_c = 3.0
        d_c = 9.96 * z_factor * r_factor * n_factor / (2 * np.pi) ** 2 * gravity
    else:
        raise ValueError("site_class must be C, D or E")

    if displacement > d_c:
        raise ValueError("displacement exceeds corner displacement")
    else:
        time = t_c * displacement / d_c
    return time


def sd_nzs(period, site_class, z_factor, r_factor, n_factor):
    """
    Returns the NZ sd_nzs value for a given Time and soil type
    :param period: float or array
    :param site_class: Either 'C', 'D' or 'E'
    output: sd_nzs: float or array
    """
    if period < 0:
        raise ValueError('Structural period is negative')
    else:
        if site_class == 'C':
            if period == 0:
                c_h = 1.33 * period ** 2
            elif period < 0.1:
                c_h = (1.33 + 1.60 * (period / 0.1)) * period ** 2
            elif period < 0.3:
                c_h = 2.93 * period ** 2
            elif period < 1.5:
                c_h = (2.0 * (0.5 / period) ** 0.75) * period ** 2
            elif period < 3.0:
                c_h = 1.32 / period * period ** 2
            else:
                c_h = 3.96
        elif site_class == 'D':
            if period == 0:
                c_h = 1.12 * period ** 2
            elif period < 0.1:
                c_h = (1.12 + 1.88 * (period / 0.1)) * period ** 2
            elif period < 0.56:
                c_h = 3.0 * period ** 2
            elif period < 1.5:
                c_h = 2.4 * (0.75 / period) ** 0.75 * period ** 2
            elif period < 3.0:
                c_h = 2.14 / period * period ** 2
            else:
                c_h = 6.42
        elif site_class == 'E':
            if period == 0:
                c_h = 1.12 * period ** 2
            elif period < 0.1:
                c_h = (1.12 + 1.88 * (period / 0.1)) * period ** 2
            elif period < 1.0:
                c_h = 3.0 * period ** 2
            elif period < 1.5:
                c_h = 3.0 / period ** 0.75 * period ** 2
            elif period < 3.0:
                c_h = 3.32 / period * period ** 2
            else:
                c_h = 9.96
        else:
            raise ValueError('Soil must be type C, D or E')
    sd = c_h * z_factor * n_factor * r_factor
    return sd


if __name__ == '__main__':
    Time = 0.7
    Soil_type = 'D'
    # Acc=Force(Time,Soil_type)


from eqsig.single import Signal, AccSignal, significant_range
from eqsig import design_spectra
from eqsig.functions import *
from eqsig.multiple import Cluster, combine_at_angle, compute_rotated
from eqsig.measures import significant_duration, calculate_peak



import numpy as np


def significant_duration(motion, dt, start=0.05, end=0.95):
    """
    Computes the significant duration using cumulative acceleration according to Trifunac and Brady (1975).
    :param motion: acceleration time series
    :param dt: time step
    :param start: threshold to start the duration
    :param end: threshold to end the duration
    :return:
    """

    acc2 = motion ** 2
    cum_acc2 = np.cumsum(acc2)
    ind2 = np.where((cum_acc2 > start * cum_acc2[-1]) & (cum_acc2 < end * cum_acc2[-1]))
    start_time = ind2[0][0] * dt
    end_time = ind2[0][-1] * dt

    return start_time, end_time


def calculate_peak(motion):
    """Calculates the peak absolute response"""
    return max(abs(min(motion)), max(motion))